// Copyright (c) 2016 Palantir Technologies. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package safejson

import (
	"bytes"
	"encoding/json"
)

// Marshal returns the JSON encoding of v encoded using the "safe" encoder.
// Unlike json.Marshal, the returned JSON bytes will not have a trailing newline.
func Marshal(v interface{}) ([]byte, error) {
	// go through Encoder to control SetEscapeHTML
	var buf bytes.Buffer
	if err := Encoder(&buf).Encode(v); err != nil {
		return nil, err
	}
	return bytes.TrimSuffix(buf.Bytes(), []byte{'\n'}), nil
}

// MarshalIndent is like Marshal but applies Indent to format the output.
func MarshalIndent(v interface{}, prefix, indent string) ([]byte, error) {
	b, err := Marshal(v)
	if err != nil {
		return nil, err
	}
	var buf bytes.Buffer
	err = json.Indent(&buf, b, prefix, indent)
	if err != nil {
		return nil, err
	}
	return buf.Bytes(), nil
}
